import json
from urllib2 import Request, urlopen, HTTPError

# debug (ahem... sort of)
from config import DEBUG_PATH
from datetime import datetime
from pprint import pprint
import os

from config import BASE_URL


def post(uri, data, base_url=BASE_URL):
    url = base_url + uri
    r = Request(url)
    r.add_header('Accept', 'application/json')
    r.add_header('Content-Type', 'application/json;charset=UTF-8')
    r.add_data(data)
    try:
        p = urlopen(r)
    except HTTPError as e:
        code = e.code
        out_data = 'read:\n%s' % e.read()
    else:
        code = p.getcode()
        out_data = p.read()
        p.close()
    if DEBUG_PATH:
        f = open(os.path.join(DEBUG_PATH, datetime.now().strftime('%Y%m%d-%H%M%S.txt')), 'a')
        f.write('url: %s\nCODE: %s\nIN\n---\n' % (url, code))
        try:
            pprint(json.loads(data), f)
        except:
            f.write('NOT JSON: %s' % data)
        f.write('\n---\nOUT\n---\n')
        try:
            pprint(json.loads(out_data), f)
        except:
            f.write('(NOT JSON)\n%s' % out_data)
        f.write('\n---\n')
        f.close()
    return code, out_data

def export_flow(flow, req, base_url=BASE_URL):
    uri = 'exportFlow?flow=%s&application=AsgTeleprocedureApa14' % flow
    data = json.dumps(req)
    return post(uri, data, base_url=base_url)

def import_flow(flow, req, base_url=BASE_URL):
    uri = 'importFlow?flow=%s&application=AsgTeleprocedureApa14' % flow
    data = json.dumps(req)
    return post(uri, data, base_url=base_url)

def conciliation_payload(config, **data):
    block = {
            "name": config['block']['name'],
            "PrimaryKey": { "key": [ config['block']['pk'] ] }
            }
    setting = []
    for affinity, afflist in config['criteria'].items():
        for aff in afflist:
            criterium = []
            for xpath, op in aff.items():
                criterium.append({
                    'key': xpath,
                    'operator': op
                    })
            setting.append({
                "affinity": affinity,
                "Criterium": criterium,
                })
    criterium = []
    for xpath, local in config['input'].items():
        criterium.append({
            'key': xpath,
            'value': data.get(local, u'')
            })
    returndata = []
    for xpath in config['output']:
        returndata.append(xpath)

    return {
        "ConciliationInputWS": {
            "Block": block,
            "Input": {
                "Settings": { "Setting": setting, }, 
                "Criteria": { "Criterium": criterium },
                },
            "Output": { "ReturnDatas": { "returnData": returndata }, },
            }
        }

def conciliation_output2dict(config, entity):
    d = {}
    for xpath in config['output']:
        x = entity
        for k in xpath.split('/'):
            x = x.get(k, {})
        d[xpath] = x or u''
    return d

def conciliation(config, base_url=BASE_URL, **data):
    uri = 'conciliation'
    name = config['block']['name'].lower()
    data = json.dumps(conciliation_payload(config, **data))
    code, out = post(uri, data, base_url=base_url)
    if code != 200:
        raise ValueError('conciliation ws: error code %d' % code)
    ret = json.loads(out)
    l = []
    count = int(ret['ConciliationOutputWS']['Results']['@count'])
    if count:
        results = ret['ConciliationOutputWS']['Results']['ResultsByAffinity']
        if type(results) is not list:
            results = [results]
        for r in results:
            affinity = r['@affinity']
            entities = r['Entities']['entity']
            if type(entities) is not list:
                entities = [entities]
            for e in entities:
                e = conciliation_output2dict(config, e)
                e['@affinity'] = affinity
                l.append(e)
    return {'results': l, 'name': name, 'count': count, 'error': False}

def referential(referential, base_url=BASE_URL,
        keys=True, order_by=False, stop_on_error=False,
        attributes=[], **filters):
    uri = 'referential?referential=%s' % referential
    data = {
        'ReferentialOptions': {
            'processKeys': keys,
            'processOrderBy': order_by,
            'stopOnError': stop_on_error,
        }}
    if filters:
        solis_filters = []
        for k,v in filters.items():
            solis_filters.append({
                'key':k,
                'value': v
                })
        data['ReferentialOptions']['Filters'] = {'Filter': solis_filters}
    if attributes:
        data['ReferentialOptions']['Attributes'] = {
                "referential": [
                    {
                        "schema": "stdr",
                        "table": referential,
                        "field": attributes,
                        }
                    ]
                }
    data = json.dumps(data)
    code, out = post(uri, data, base_url=base_url)
    if code != 200:
        raise ValueError('referential ws: error code %d' % code)
    ret = json.loads(out)
    l = []
    count = int(ret['ReferentialOutputWS']['Entries']['@count'])
    name = ret['ReferentialOutputWS']['Entries']['@name']
    if count:
        entries = ret['ReferentialOutputWS']['Entries']['Entry']
        if type(entries) is not list:
            entries = [entries]
        l += entries
    return {'results': l, 'name': name, 'count': count, 'error': False}

