import json

from django.core.urlresolvers import reverse
from django.http import Http404
from django.shortcuts import redirect
from django.views.generic.base import View, RedirectView
from django.views.generic.detail import SingleObjectMixin, DetailView
from django.views.generic.edit import CreateView, UpdateView, DeleteView

from passerelle.utils import to_json
from passerelle import utils

from passerelle.base.views import ResourceView

from .models import Concerto
from .forms import ConcertoForm


class ConcertoDetailView(DetailView):
    model = Concerto


class ConcertoCreateView(CreateView):
    model = Concerto
    form_class = ConcertoForm
    template_name = 'passerelle/manage/service_form.html'


class ConcertoUpdateView(UpdateView):
    model = Concerto
    form_class = ConcertoForm
    template_name = 'passerelle/manage/service_form.html'


class ConcertoDeleteView(DeleteView):
    model = Concerto
    template_name = 'passerelle/manage/service_confirm_delete.html'

    def get_success_url(self):
        return reverse('manage-home')

class FamilyLink(View, SingleObjectMixin):
    model = Concerto

    @utils.protected_api('can_use_connector')
    @to_json('api')
    def get(self, request, slug, family_id, secret, *args, **kwargs):
        return self.get_object().family_link(family_id, secret)

class FamilyData(View, SingleObjectMixin):
    model = Concerto

    @utils.protected_api('can_use_connector')
    @to_json('api')
    def get(self, request, slug, nameid):
        data_filter = request.GET.get('filter', None)
        return self.get_object().family_data(nameid, data_filter)

class LinkToFamily(View, SingleObjectMixin):
    model = Concerto

    @utils.protected_api('can_use_connector')
    @to_json('api')
    def get(self, request, slug, nameid, family_id):
        return self.get_object().link_to_family(nameid, family_id)

class UnlinkFromFamily(View, SingleObjectMixin):
    model = Concerto

    @utils.protected_api('can_use_connector')
    @to_json('api')
    def get(self, request, slug, nameid):
        return self.get_object().unlink_from_family(nameid)

class FamilyInvoices(View, SingleObjectMixin):
    model = Concerto

    @utils.protected_api('can_use_connector')
    @to_json('api')
    def get(self, request, slug, nameid):
        status = request.GET.get('status', 'all')
        return self.get_object().family_invoices(nameid, status)

class InvoiceView(View, SingleObjectMixin):
    model = Concerto

    @utils.protected_api('can_use_connector')
    @to_json('api')
    def get(self, request, slug, invoice_id, invoice_hash):
        return self.get_object().get_invoice(invoice_id, invoice_hash)

    @utils.protected_api('can_use_connector')
    @to_json('api')
    def post(self, request, slug, invoice_id, invoice_hash):
        invoice_status = request.POST.get('status')
        return self.get_object().update_invoice(invoice_id, invoice_hash, invoice_status)
