import hashlib
import datetime

from .models import ApiUser
from .signature import check_query

import logging
logger = logging.getLogger(__name__)

def get_apiuser_with_signature(orig, query):
    """
    Returns ApiUser(username=orig) if the signature in the query string matches
    """
    try:
        apiuser = ApiUser.objects.get(keytype='SIGN', username=orig)
    except ApiUser.DoesNotExist:
        logger.warning('unknown user')
        return None
    try:
        if check_query(query, apiuser.key):
            return apiuser
    except:
        pass
    logger.warning('invalid signature')
    return None


class SearchApiUser(object):
    def process_request(self, request):
        request.apiuser = None

        if 'orig' in request.GET and 'signature' in request.GET:
            request.apiuser = get_apiuser_with_signature(
                    orig=request.GET['orig'],
                    query=request.META['QUERY_STRING'])

        elif 'apikey' in request.GET:
            try:
                request.apiuser = ApiUser.objects.get(keytype='API',
                        key=request.GET['apikey'])
            except ApiUser.DoesNotExist:
                logger.warning('unknown apikey')

        elif request.META.has_key('HTTP_AUTHORIZATION'):
            (scheme, param) = request.META['HTTP_AUTHORIZATION'].split(' ',1)
            if scheme.lower() == 'basic':
                username, password = param.strip().decode('base64').split(':',1)
                try:
                    request.apiuser = ApiUser.objects.get(keytype='SIGN',
                            username=username, key=password)
                except ApiUser.DoesNotExist:
                    logger.warning('HTTP_AUTHORIZATION: unknown user/password')

