from django.contrib.auth import logout as auth_logout
from django.contrib.auth import views as auth_views
from django.http import HttpResponseRedirect
from django.views.generic.base import TemplateView
from django.conf import settings
from django.db import models
from django.shortcuts import resolve_url

try:
    from mellon.utils import get_idps
except ImportError:
    get_idps = lambda: []

from passerelle.base.models import BaseResource


def get_all_apps():
    # TODO: once the legacy connectors are ported, the test on get_icon_class
    # won't be necessary anymore.
    return [x for x in models.get_models() if issubclass(x, BaseResource) and \
            hasattr(x, 'get_icon_class')]

def login(request, *args, **kwargs):
    if any(get_idps()):
        if not 'next' in request.GET:
            return HttpResponseRedirect(resolve_url('mellon_login'))
        return HttpResponseRedirect(resolve_url('mellon_login') + '?next=' + request.GET.get('next'))
    return auth_views.login(request, *args, **kwargs)

def logout(request, next_page=None):
    if any(get_idps()):
        return HttpResponseRedirect(resolve_url('mellon_logout'))
    auth_logout(request)
    if next_page is not None:
        next_page = resolve_url(next_page)
    else:
        next_page = '/'
    return HttpResponseRedirect(next_page)


class HomePageView(TemplateView):
    template_name = 'passerelle/homepage.html'

    def get_context_data(self, **kwargs):
        context = super(HomePageView, self).get_context_data(**kwargs)
        # get all app instances
        context['apps'] = []
        for app in get_all_apps():
            context['apps'].extend(app.objects.all())
        return context


class ManageView(TemplateView):
    template_name = 'passerelle/manage.html'

    def get_context_data(self, **kwargs):
        context = super(ManageView, self).get_context_data(**kwargs)
        # get all app instances
        context['apps'] = []
        for app in get_all_apps():
            context['apps'].extend(app.objects.all())
        return context


class ManageAddView(TemplateView):
    template_name = 'passerelle/manage_add.html'

    def get_context_data(self, **kwargs):
        context = super(ManageAddView, self).get_context_data(**kwargs)
        context['apps'] = get_all_apps()
        return context


# legacy
LEGACY_APPS_PATTERNS = {
    'datasources': {'url': 'data', 'name': 'Data Sources'},
    'repost': {'url': 'repost', 'name': 'Re-Post'},
    'register': {'url': r'register', 'name': 'Register'},
    'queue': {'url': r'queue', 'name': 'Queues'},
}

class LegacyPageView(TemplateView):
    template_name = 'passerelle/legacy.html'

    def get_context_data(self, **kwargs):
        context = super(LegacyPageView, self).get_context_data(**kwargs)
        # legacy apps (categories)
        context['legacy_apps'] = []
        for app in (app for app in settings.INSTALLED_APPS
                if app.startswith('passerelle.')
                    and app.split('.')[1] in LEGACY_APPS_PATTERNS):
            context['legacy_apps'].append(LEGACY_APPS_PATTERNS[app.split('.')[1]])
        return context
