import json

from django.core.urlresolvers import reverse
from django.http import Http404
from django.shortcuts import redirect
from django.views.generic.base import View, RedirectView
from django.views.generic.detail import SingleObjectMixin, DetailView
from django.views.generic.edit import CreateView, UpdateView, DeleteView

from passerelle import utils
from passerelle.base.views import ResourceView

from .models import Pastell
from .forms import PastellForm, PastellTypeForm, PastellFieldsForm

class PostView(View, SingleObjectMixin):
    model = Pastell

    @utils.protected_api('can_post_document')
    def post(self, request, *args, **kwargs):
        data = json.loads(request.body)
        self.get_object().create_document(data)
        result = {'result': 'ok'}
        return utils.response_for_json(request, result)


class PastellDetailView(DetailView):
    model = Pastell

    def get_context_data(self, **kwargs):
        context = super(PastellDetailView, self).get_context_data(**kwargs)
        if self.object.document_type and self.object.document_fields:
            context['fields'] = []
            for field in self.object.get_document_fields().keys():
                if self.object.document_fields.get(field):
                    context['fields'].append({'key': field,
                        'value': self.object.document_fields.get(field)})
        return context


class PastellCreateView(CreateView):
    model = Pastell
    form_class = PastellForm
    template_name = 'passerelle/manage/service_form.html'


class PastellUpdateView(UpdateView):
    model = Pastell
    form_class = PastellForm
    template_name = 'passerelle/manage/service_form.html'


class PastellDeleteView(DeleteView):
    model = Pastell
    template_name = 'passerelle/manage/service_confirm_delete.html'

    def get_success_url(self):
        return reverse('manage-home')


class PastellTypeView(UpdateView):
    model = Pastell
    form_class = PastellTypeForm
    template_name = 'passerelle/manage/service_form.html'


class PastellFieldsView(UpdateView):
    model = Pastell
    form_class = PastellFieldsForm
    template_name = 'passerelle/manage/service_form.html'


